@@ -1,4 +1,4 @@
-# $Id: Request.pm 178 2005-06-22 16:22:17Z rcaputo $
+# $Id: Request.pm 196 2005-07-28 03:27:36Z rcaputo $
 
 package POE::Component::Client::HTTP::Request;
 use strict;
@@ -140,7 +140,6 @@ sub new {
     undef,              # unused
     0,                  # REQ_OCTETS_GOT
     undef,              # REQ_TIMER
-    #"\x0D\x0A",         # REQ_NEWLINE
     $progress,          # REQ_PROG_POSTBACK
     $using_proxy,       # REQ_USING_PROXY
     $host,              # REQ_HOST
@@ -188,6 +187,37 @@ sub return_response {
   $self->[REQ_BUFFER] = '';
 }
 
+sub add_eof {
+  my ($self) = @_;
+
+  return if ($self->[REQ_STATE] & RS_POSTED);
+
+  unless (defined $self->[REQ_RESPONSE]) {
+    # XXX I don't know if this is actually used
+    $self->error(400, "incomplete response " . $self->[REQ_ID]);
+    return;
+  }
+
+  # RFC 2616: "If a message is received with both a Transfer-Encoding
+  # header field and a Content-Length header field, the latter MUST be
+  # ignored."
+  if (
+    defined $self->[REQ_RESPONSE]->content_length and
+    not defined $self->[REQ_RESPONSE]->header("Transfer-Encoding") and
+    $self->[REQ_OCTETS_GOT] < $self->[REQ_RESPONSE]->content_length
+  ) {
+    DEBUG and warn(
+      "got " . $self->[REQ_OCTETS_GOT] . " of " .
+      $self->[REQ_RESPONSE]->content_length
+    );
+    $self->error(400, "incomplete response " . $self->[REQ_ID]);
+  }
+  else {
+    $self->[REQ_STATE] |= RS_DONE;
+    $self->return_response();
+  }
+}
+
 sub add_content {
   my ($self, $data) = @_;
 
@@ -285,6 +315,7 @@ sub timer {
 
   # do it this way so we can set REQ_TIMER to undef
   if (@_ == 2) {
+    die "overwriting timer $self->[REQ_TIMER]" if $self->[REQ_TIMER];
     $self->[REQ_TIMER] = $timer;
   }
   return $self->[REQ_TIMER];
@@ -314,6 +345,7 @@ sub remove_timeout {
     my $kernel = $POE::Kernel::poe_kernel;
     DEBUG and warn "REQ: Removing timer $alarm_id";
     $kernel->alarm_remove($alarm_id);
+    $self->[REQ_TIMER] = undef;
   }
 }
 
